/*
Dot Zip
by Joo Won Park
www.joowonpark.net
@joowonmusic
Mirror Tempo
A motif is repeated from tempo1 to tempo2 while the reverse of that melody is repeated with opposite tempo change
*/

//1. SynthDefs
(
SynthDef("TheSound",{
	arg amp,freq,dur,pan;
	var sound,env,mix;
	sound=Blip.ar([freq,Rand(0.99,1.01)*freq],Rand(3,10));
	env=Env.perc(0.03,0.95,Rand(-4.0,-3.0));
	env=EnvGen.ar(env,timeScale:dur,doneAction:2);
	mix=Pan2.ar(sound.sum,pan)*env*amp;
	Out.ar(0,mix)

}).add;
);

//2.Functions
(
~key=58;
~motif=[0,2,4,7,9]+~key;

~mirror={
	arg size,bpm1,bpm2,amp,dur,pan;
	var formula,pitch;
	formula=60/(Array.interpolation(size,bpm1,bpm2));
	pitch=~motif.scramble;
	Synth("TheSound",[\amp,amp*1.2,\freq,(~motif.choose-12).midicps,\dur,dur*rrand(7.0,8.5),\pan,0]);

	Routine({

		size.do{
			arg count;
			Synth("TheSound",[\amp,rrand(0.8,1.0)*amp,
				\freq,(pitch.wrapAt(count)).midicps,
				\dur,rrand(0.8,1.3)*dur,\pan,pan]);
			formula.at(count).wait;
		}
	}).play;

	Routine({
		size.do{
			arg count;
			Synth("TheSound",[\amp,rrand(0.8,1.0)*amp,
				\freq,(pitch.reverse.wrapAt(count)+12).midicps,
				\dur,rrand(0.8,1.3)*dur,\pan,pan.neg]);
			formula.reverse.at(count).wait;
		}
	}).play;
};
);


//3. Perform
s.recSampleFormat_('int24');
s.record;
s.stopRecording
(
//~mirror.(size,bpm1,bpm2,amp,dur,pan;)
~motif=[0,2,4,6,8,10,12]+~key;
//~motif=[0,2,4,7,9]+~key;
Routine({
	3.do{
		//~mirror.(repeat numbers,bpm1,bpm2,amplitude,noteduration,pan)
		~mirror.(rrand(100,120),rrand(400,600),rrand(50,200),
			rrand(0.03,0.05),rrand(0.5,2.0),rrand(-1.0,1.0));
		rrand(7,10.0).wait;
	};
}).play
);


